/* ObjectFile.java */
package jwiley.effective;

import java.io.*;
import java.util.*;

public class ObjectFile
{
   RandomAccessFile dataFile;
   String sFileName;

   public ObjectFile(String sName) throws IOException
   {
      sFileName = sName;
      dataFile = new RandomAccessFile(sName, "rw");
   }

   // returns file position object was written to.
   public synchronized long writeObject(Serializable obj) throws IOException
   {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(obj);
      oos.flush();

      int datalen = baos.size();

      // append record
      long pos = dataFile.length();
      dataFile.seek(pos);

      // write the length of the output
      dataFile.writeInt(datalen);
      dataFile.write(baos.toByteArray());

      baos = null; oos = null;

      return pos;
   }

   public synchronized void rewriteObject(long pos, 
                            Serializable obj) throws IOException
   {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(obj);
      oos.flush();

      int datalen = baos.size();

      // insert record
      dataFile.seek(pos);

      // write the length of the output
      dataFile.writeInt(datalen);
      dataFile.write(baos.toByteArray());

      baos = null; oos = null;
   }

   public synchronized void rewriteObject(long pos, 
                       byte [] byteArray) throws IOException
   {
      int datalen = byteArray.length;

      // insert record
      dataFile.seek(pos);

      // write the length of the output
      dataFile.writeInt(datalen);
      dataFile.write(byteArray);
   }

   // get the current object length - restore or move?
   public synchronized int getObjectLength(long lPos) throws IOException
   {
      dataFile.seek(lPos);
      return dataFile.readInt();
   }

   public synchronized Object readObject(long lPos) 
                throws IOException,ClassNotFoundException
   {
      dataFile.seek(lPos);
      int datalen = dataFile.readInt();
      if (datalen > dataFile.length())
         throw new IOException("Data file is corrupted. datalen: " 
                                + datalen);
      byte [] data = new byte[datalen];
      dataFile.readFully(data);

      ByteArrayInputStream bais = new ByteArrayInputStream(data);
      ObjectInputStream ois = new ObjectInputStream(bais);
      Object o = ois.readObject();

      bais = null;
      ois = null;
      data = null;

      return o;
   }

   public long length() throws IOException
   {
      return dataFile.length();
   }

   public void close() throws IOException
   {
      dataFile.close();
   }

   public static void main(String args[])
   {
      try
      {
         ObjectFile of = new ObjectFile("tmp.dat");
         int iSize = 10;
         long [] positions = new long[iSize];
         for (int i = 0; i < iSize; i++)
            positions[i] = of.writeObject(new Integer(i));

         System.out.println("Wrote to tmp.dat");

         for (int i = 0; i < iSize; i++)
            System.out.println(of.readObject(positions[i]));

         of.close();
      } catch (Exception e)
        {
           e.printStackTrace();
        }
   }
} // end of ObjectFile class
